/*
 * Copyright (c) 2009, Mason Green (zzzzrrr)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * * Neither the name of the polygonal nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
module openmelee.ship;

import tango.io.Stdout: Stdout;

import tango.util.container.LinkedList : LinkedList;
import blaze.dynamics.bzBody : bzBody;
import blaze.collision.shapes.bzShape : bzShape;
import blaze.common.bzMath: bzVec2, bzCross;
import blaze.bzWorld : bzWorld;
import blaze.dynamics.forces.bzAttractor: bzAttractor;

alias LinkedList!(bzShape) ShapeList;

abstract class Ship
{
    bzWorld world;
    bzBody rBody;
    ShapeList shapes;
    bzVec2 engineForce;
    bzVec2 turnForce;
    bzVec2 leftTurnPoint;
    bzVec2 rightTurnPoint;

    this(bzWorld world) {
        this.world = world;
        shapes = new ShapeList;
    }

    void thrust() {
        rBody.force += engineForce.rotate(rBody.angle);
    }

    void turnLeft() {
        rBody.torque += bzCross(leftTurnPoint.rotate(rBody.angle), 
                                turnForce.rotate(rBody.angle));
    }
    
    void turnRight() {
        rBody.torque += bzCross(rightTurnPoint.rotate(rBody.angle), 
                                turnForce.rotate(rBody.angle));
    }
    
    void setGravity() {
        
        float minRadius = 1;
        float maxRadius = 30;
        float strength = 4;
        bzVec2 center = bzVec2(0,0);
        auto attractor = new bzAttractor(rBody, center, strength, minRadius, maxRadius);
        world.addForce(attractor);
    }
}
